package main

import (
	"encoding/json"
	"fmt"
	"log"
	"io/ioutil"
	"os"
	"regexp"

	"srs.epita.fr/fic-server/libfic"
)

func validTeamName(name string) bool {
	match, err := regexp.MatchString("^[A-Za-z0-9 àéèêëîïôùûü_-]{1,32}$", name)
	return err == nil && match
}

func treatRename(pathname string, team fic.Team) {
	var keys map[string]string

	if cnt_raw, err := ioutil.ReadFile(pathname); err != nil {
		log.Println("[ERR]", err)
	} else if err := json.Unmarshal(cnt_raw, &keys); err != nil {
		log.Println("[ERR]", err)
	} else if validTeamName(keys["newName"]) {
		team.Name = keys["newName"]
		if _, err := team.Update(); err != nil {
			log.Println("[WRN] Unable to change team name:", err)
		}
		if err := genTeamMyFile(team); err != nil {
			log.Println("my-", team.Id, ".json generation error: ", err)
		}
		if _, err := fic.NewEvent(fmt.Sprintf("Souhaitons bonne chance à l'équipe <strong>%s</strong> qui vient de nous rejoindre !", team.Name), "alert-info"); err != nil {
			log.Println("[WRN] Unable to create event:", err)
		}
		if err := genEventsFile(); err != nil {
			log.Println("events.json generation error: ", err)
		}
		if err := os.Remove(pathname); err != nil {
			log.Println("[ERR]", err)
		}
	}
}
