package fic

import (
	"fmt"
)

// exportedExercice is a structure representing a challenge, as exposed to players.
type exportedExercice struct {
	Title  string   `json:"title"`
	URLId  string   `json:"urlid"`
	Tags   []string `json:"tags"`
	Gain   int64    `json:"gain"`
	Coeff  float64  `json:"curcoeff"`
	Solved int64    `json:"solved"`
	Tried  int64    `json:"tried"`
}

// exportedTheme is a structure representing a Theme, as exposed to players.
type exportedTheme struct {
	Name      string                      `json:"name"`
	URLId     string                      `json:"urlid"`
	Authors   string                      `json:"authors"`
	Intro     string                      `json:"intro"`
	Exercices map[string]exportedExercice `json:"exercices"`
}

// Exportedthemes exports themes from the database, to be displayed to players.
func ExportThemes() (interface{}, error) {
	if themes, err := GetThemes(); err != nil {
		return nil, err
	} else {
		ret := map[string]exportedTheme{}
		for _, theme := range themes {
			if exercices, err := theme.GetExercices(); err != nil {
				return nil, err
			} else {
				exos := map[string]exportedExercice{}
				for _, exercice := range exercices {
					tags, _ := exercice.GetTags()
					exos[fmt.Sprintf("%d", exercice.Id)] = exportedExercice{
						exercice.Title,
						exercice.URLId,
						tags,
						exercice.Gain,
						exercice.Coefficient,
						exercice.SolvedCount(),
						exercice.TriedTeamCount(),
					}
				}
				ret[fmt.Sprintf("%d", theme.Id)] = exportedTheme{
					theme.Name,
					theme.URLId,
					theme.Authors,
					theme.Intro,
					exos,
				}
			}
		}

		return ret, nil
	}
}
